@@ -1,3 +1,17 @@
+# Copyright 2001-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::Test;
 
 use strict;
@@ -8,7 +22,7 @@ use Exporter ();
 use Config;
 use Apache::TestConfig ();
 
-use vars qw(@ISA @EXPORT $VERSION %SubTests @SkipReasons);
+use vars qw(@ISA @EXPORT %EXPORT_TAGS $VERSION %SubTests @SkipReasons);
 
 @ISA = qw(Exporter);
 @EXPORT = qw(ok skip sok plan have have_lwp have_http11
@@ -16,7 +30,13 @@ use vars qw(@ISA @EXPORT $VERSION %SubTests @SkipReasons);
              have_min_apache_version have_apache_version have_perl 
              have_min_perl_version have_min_module_version
              have_threads under_construction have_apache_mpm);
-$VERSION = '1.08';
+
+# everything but ok(), skip(), and plan() - Test::More provides these
+my @test_more_exports = grep { ! /^(ok|skip|plan)$/ } @EXPORT;
+
+%EXPORT_TAGS = (withtestmore => \@test_more_exports);
+
+$VERSION = '1.10';
 
 %SubTests = ();
 @SkipReasons = ();
@@ -716,6 +736,28 @@ are returned.
 
 =back
 
+=head1 Test::More Integration
+
+There are a few caveats if you want to use I<Apache::Test> with 
+I<Test::More> instead of the default I<Test> backend.  The first is
+that I<Test::More> requires you to use its own C<plan()> function
+and not the one that ships with I<Apache::Test>.  I<Test::More> also
+defines C<ok()> and C<skip()> functions that are different, and 
+simply C<use>ing both modules in your test script will lead to redefined
+warnings for these subroutines.
+
+To assist I<Test::More> users we have created a special I<Apache::Test>
+import tag, C<:withtestmore>, which will export all of the standard
+I<Apache::Test> symbols into your namespace except the ones that collide
+with I<Test::More>.
+
+    use Apache::Test qw(:withtestmore);
+    use Test::More;
+
+    plan tests => 1;           # Test::More::plan()
+
+    ok ('yes', 'testing ok');  # Test::More::ok()
+
 =head1 Apache::TestToString Class
 
 The I<Apache::TestToString> class is used to capture I<Test.pm> output
