@@ -1,3 +1,17 @@
+# Copyright 2001-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::TestHarness;
 
 use strict;
@@ -49,7 +63,8 @@ sub skip {
 sub run_t {
     my($self, $file) = @_;
     my $ran = 0;
-    my $cmd = "$^X -Mlib=../Apache-Test/lib $file";
+    my $lib = catfile Apache::Test::vars('top_dir'), qw(Apache-Test lib);
+    my $cmd = qq[$^X -Mlib="$lib" $file];
 
     my $h = Symbol::gensym();
     open $h, "$cmd|" or die "open $cmd: $!";
