@@ -90,8 +90,10 @@ sub generate_script {
 
     $body .= Apache::TestConfig->modperl_2_inc_fixup;
 
-    if (@Apache::TestMM::Argv) {
-        $body .= "\n\%Apache::TestConfig::Argv = qw(@Apache::TestMM::Argv);\n";
+    my %args = @Apache::TestMM::Argv;
+    while (my($k, $v) = each %args) {
+        $v =~ s/\|/\\|/g;
+        $body .= "\n\$Apache::TestConfig::Argv{'$k'} = q|$v|;\n";
     }
 
     my $in = Symbol::gensym();
@@ -164,12 +166,13 @@ its mess.
   Apache::TestMM::filter_args();
   WriteMakefile(...);
 
-When C<WriteMakefile()> is called it parses C<@ARGV>, hoping to find special
-options like C<PREFIX=/home/stas/perl>. C<Apache::Test> accepts a lot of
-options of its own. When C<Apache::TestMM::filter_args()> is called, it
-removes any C<Apache::Test>-specific options from C<@ARGV> and stores them
-internally, so when C<WriteMakefile()> is called they aren't in C<@ARGV> and
-thus won't be processed by C<WriteMakefile()>.
+When C<WriteMakefile()> is called it parses C<@ARGV>, hoping to find
+special options like C<PREFIX=/home/stas/perl>. C<Apache::Test>
+accepts a lot of configuration options of its own. When
+C<Apache::TestMM::filter_args()> is called, it removes any
+C<Apache::Test>-specific options from C<@ARGV> and stores them
+internally, so when C<WriteMakefile()> is called they aren't in
+C<@ARGV> and thus won't be processed by C<WriteMakefile()>.
 
 The options can be set when F<Makefile.PL> is called:
 
@@ -191,6 +194,20 @@ file. In our example, the autogenerated F<t/TEST> will include:
 
 which is going to be used by the C<Apache::Test> runtime.
 
+The other frequently used options are: C<-httpd>, telling where to
+find the httpd (usually when the C<-apxs> option is not used),
+C<-libmodperl> to use a specific mod_perl shared object (if your
+mod_perl is built as DSO), C<-maxclients> to change the default number
+of the configured C<MaxClients> directive, C<-port> to start the
+server on a specific port, etc. To get the complete list of available
+configuration options and their purpose and syntax, run:
+
+  % perl -MApache::TestConfig -le 'Apache::TestConfig::usage()'
+
+You may wish to document some of these in your application's F<README>
+file, especially the C<-apxs> and C<-httpd> options.
+
+
 =head2 C<generate_script>
 
   Apache::TestMM::generate_script('t/TEST');
